aboutsummaryrefslogtreecommitdiffstats
diff options
context:
space:
mode:
-rw-r--r--Makefile5
-rw-r--r--cache.c51
-rw-r--r--cgit.c37
-rw-r--r--cgit.css6
-rw-r--r--cgit.h12
-rw-r--r--cgit.js68
-rw-r--r--cgitrc.5.txt30
-rw-r--r--cmd.c18
-rwxr-xr-xfilters/commit-links.sh2
-rwxr-xr-xfilters/html-converters/md2html7
m---------git0
-rw-r--r--html.c2
-rw-r--r--parsing.c5
-rw-r--r--robots.txt1
-rw-r--r--shared.c12
-rwxr-xr-xtests/setup.sh14
-rwxr-xr-xtests/t0001-validate-git-versions.sh8
-rwxr-xr-xtests/t0105-commit.sh2
-rwxr-xr-xtests/t0107-snapshot.sh125
-rwxr-xr-xtests/t0109-gitconfig.sh2
-rw-r--r--ui-atom.c34
-rw-r--r--ui-blame.c37
-rw-r--r--ui-blob.c15
-rw-r--r--ui-commit.c15
-rw-r--r--ui-diff.c12
-rw-r--r--ui-log.c55
-rw-r--r--ui-patch.c2
-rw-r--r--ui-plain.c14
-rw-r--r--ui-repolist.c2
-rw-r--r--ui-shared.c107
-rw-r--r--ui-snapshot.c38
-rw-r--r--ui-stats.c51
-rw-r--r--ui-tag.c6
-rw-r--r--ui-tree.c49
34 files changed, 582 insertions, 262 deletions
diff --git a/Makefile b/Makefile
index b51de6f..ad825be 100644
--- a/Makefile
+++ b/Makefile
@@ -1,6 +1,6 @@
all::
-CGIT_VERSION = v1.2.2
+CGIT_VERSION = v1.2.3
CGIT_SCRIPT_NAME = cgit.cgi
CGIT_SCRIPT_PATH = /var/www/htdocs/cgit
CGIT_DATA_PATH = $(CGIT_SCRIPT_PATH)
@@ -14,7 +14,7 @@ htmldir = $(docdir)
pdfdir = $(docdir)
mandir = $(prefix)/share/man
SHA1_HEADER = <openssl/sha.h>
-GIT_VER = 2.25.0
+GIT_VER = 2.39.0
GIT_URL = https://www.kernel.org/pub/software/scm/git/git-$(GIT_VER).tar.xz
INSTALL = install
COPYTREE = cp -r
@@ -87,6 +87,7 @@ install: all
$(INSTALL) -m 0755 cgit $(DESTDIR)$(CGIT_SCRIPT_PATH)/$(CGIT_SCRIPT_NAME)
$(INSTALL) -m 0755 -d $(DESTDIR)$(CGIT_DATA_PATH)
$(INSTALL) -m 0644 cgit.css $(DESTDIR)$(CGIT_DATA_PATH)/cgit.css
+ $(INSTALL) -m 0644 cgit.js $(DESTDIR)$(CGIT_DATA_PATH)/cgit.js
$(INSTALL) -m 0644 cgit.png $(DESTDIR)$(CGIT_DATA_PATH)/cgit.png
$(INSTALL) -m 0644 favicon.ico $(DESTDIR)$(CGIT_DATA_PATH)/favicon.ico
$(INSTALL) -m 0644 robots.txt $(DESTDIR)$(CGIT_DATA_PATH)/robots.txt
diff --git a/cache.c b/cache.c
index 2c70be7..1c843ba 100644
--- a/cache.c
+++ b/cache.c
@@ -85,40 +85,45 @@ static int close_slot(struct cache_slot *slot)
/* Print the content of the active cache slot (but skip the key). */
static int print_slot(struct cache_slot *slot)
{
+ off_t off;
#ifdef HAVE_LINUX_SENDFILE
- off_t start_off;
- int ret;
+ off_t size;
+#endif
+
+ off = slot->keylen + 1;
- start_off = slot->keylen + 1;
+#ifdef HAVE_LINUX_SENDFILE
+ size = slot->cache_st.st_size;
do {
- ret = sendfile(STDOUT_FILENO, slot->cache_fd, &start_off,
- slot->cache_st.st_size - start_off);
+ ssize_t ret;
+ ret = sendfile(STDOUT_FILENO, slot->cache_fd, &off, size - off);
if (ret < 0) {
if (errno == EAGAIN || errno == EINTR)
continue;
+ /* Fall back to read/write on EINVAL or ENOSYS */
+ if (errno == EINVAL || errno == ENOSYS)
+ break;
return errno;
}
- return 0;
+ if (off == size)
+ return 0;
} while (1);
-#else
- ssize_t i, j;
+#endif
- i = lseek(slot->cache_fd, slot->keylen + 1, SEEK_SET);
- if (i != slot->keylen + 1)
+ if (lseek(slot->cache_fd, off, SEEK_SET) != off)
return errno;
do {
- i = j = xread(slot->cache_fd, slot->buf, sizeof(slot->buf));
- if (i > 0)
- j = xwrite(STDOUT_FILENO, slot->buf, i);
- } while (i > 0 && j == i);
-
- if (i < 0 || j != i)
- return errno;
- else
- return 0;
-#endif
+ ssize_t ret;
+ ret = xread(slot->cache_fd, slot->buf, sizeof(slot->buf));
+ if (ret < 0)
+ return errno;
+ if (ret == 0)
+ return 0;
+ if (write_in_full(STDOUT_FILENO, slot->buf, ret) < 0)
+ return errno;
+ } while (1);
}
/* Check if the slot has expired */
@@ -401,12 +406,12 @@ int cache_process(int size, const char *path, const char *key, int ttl,
static char *sprintftime(const char *format, time_t time)
{
static char buf[64];
- struct tm *tm;
+ struct tm tm;
if (!time)
return NULL;
- tm = gmtime(&time);
- strftime(buf, sizeof(buf)-1, format, tm);
+ gmtime_r(&time, &tm);
+ strftime(buf, sizeof(buf)-1, format, &tm);
return buf;
}
diff --git a/cgit.css b/cgit.css
index d4aadbf..1b848cf 100644
--- a/cgit.css
+++ b/cgit.css
@@ -363,6 +363,10 @@ div#cgit table.blame td.lines > div > pre {
top: 0;
}
+div#cgit table.blame .oid {
+ font-size: 100%;
+}
+
div#cgit table.bin-blob {
margin-top: 0.5em;
border: solid 1px black;
@@ -561,7 +565,7 @@ div#cgit table.diff td div.del {
color: red;
}
-div#cgit .sha1 {
+div#cgit .oid {
font-family: monospace;
font-size: 90%;
}
diff --git a/cgit.h b/cgit.h
index 7ec46b4..ddd2ccb 100644
--- a/cgit.h
+++ b/cgit.h
@@ -14,7 +14,7 @@
#include <tag.h>
#include <diff.h>
#include <diffcore.h>
-#include <argv-array.h>
+#include <strvec.h>
#include <refs.h>
#include <revision.h>
#include <log-tree.h>
@@ -25,6 +25,7 @@
#include <utf8.h>
#include <notes.h>
#include <graph.h>
+#include <inttypes.h>
/* Add isgraph(x) to Git's sane ctype support (see git-compat-util.h) */
#undef isgraph
@@ -164,7 +165,7 @@ struct reflist {
struct cgit_query {
int has_symref;
- int has_sha1;
+ int has_oid;
int has_difftype;
char *raw;
char *repo;
@@ -172,8 +173,8 @@ struct cgit_query {
char *search;
char *grep;
char *head;
- char *sha1;
- char *sha2;
+ char *oid;
+ char *oid2;
char *path;
char *name;
char *url;
@@ -195,7 +196,6 @@ struct cgit_config {
char *cache_root;
char *clone_prefix;
char *clone_url;
- char *css;
char *favicon;
char *footer;
char *head_include;
@@ -206,6 +206,7 @@ struct cgit_config {
char *module_link;
char *project_list;
struct string_list readme;
+ struct string_list css;
char *robots;
char *root_title;
char *root_desc;
@@ -264,6 +265,7 @@ struct cgit_config {
int branch_sort;
int commit_sort;
struct string_list mimetypes;
+ struct string_list js;
struct cgit_filter *about_filter;
struct cgit_filter *commit_filter;
struct cgit_filter *source_filter;
diff --git a/cgit.js b/cgit.js
new file mode 100644
index 0000000..df3ad4e
--- /dev/null
+++ b/cgit.js
@@ -0,0 +1,68 @@
+/* cgit.js: javacript functions for cgit
+ *
+ * Copyright (C) 2006-2018 cgit Development Team <cgit@lists.zx2c4.com>
+ *
+ * Licensed under GNU General Public License v2
+ * (see COPYING for full license text)
+ */
+
+(function () {
+
+/* This follows the logic and suffixes used in ui-shared.c */
+
+var age_classes = [ "age-mins", "age-hours", "age-days", "age-weeks", "age-months", "age-years" ];
+var age_suffix = [ "min.", "hours", "days", "weeks", "months", "years", "years" ];
+var age_next = [ 60, 3600, 24 * 3600, 7 * 24 * 3600, 30 * 24 * 3600, 365 * 24 * 3600, 365 * 24 * 3600 ];
+var age_limit = [ 7200, 24 * 7200, 7 * 24 * 7200, 30 * 24 * 7200, 365 * 25 * 7200, 365 * 25 * 7200 ];
+var update_next = [ 10, 5 * 60, 1800, 24 * 3600, 24 * 3600, 24 * 3600, 24 * 3600 ];
+
+function render_age(e, age) {
+ var t, n;
+
+ for (n = 0; n < age_classes.length; n++)
+ if (age < age_limit[n])
+ break;
+
+ t = Math.round(age / age_next[n]) + " " + age_suffix[n];
+
+ if (e.textContent != t) {
+ e.textContent = t;
+ if (n == age_classes.length)
+ n--;
+ if (e.className != age_classes[n])
+ e.className = age_classes[n];
+ }
+}
+
+function aging() {
+ var n, next = 24 * 3600,
+ now_ut = Math.round((new Date().getTime() / 1000));
+
+ for (n = 0; n < age_classes.length; n++) {
+ var m, elems = document.getElementsByClassName(age_classes[n]);
+
+ if (elems.length && update_next[n] < next)
+ next = update_next[n];
+
+ for (m = 0; m < elems.length; m++) {
+ var age = now_ut - elems[m].getAttribute("data-ut");
+
+ render_age(elems[m], age);
+ }
+ }
+
+ /*
+ * We only need to come back when the age might have changed.
+ * Eg, if everything is counted in hours already, once per
+ * 5 minutes is accurate enough.
+ */
+
+ window.setTimeout(aging, next * 1000);
+}
+
+document.addEventListener("DOMContentLoaded", function() {
+ /* we can do the aging on DOM content load since no layout dependency */
+ aging();
+}, false);
+
+})();
diff --git a/cgitrc.5.txt b/cgitrc.5.txt
index ba77826..6f3e952 100644
--- a/cgitrc.5.txt
+++ b/cgitrc.5.txt
@@ -126,7 +126,8 @@ commit-sort::
css::
Url which specifies the css document to include in all cgit pages.
- Default value: "/cgit.css".
+ Default value: "/cgit.css". May be given multiple times, each
+ css URL path is added in the head section of the document in turn.
email-filter::
Specifies a command which will be invoked to format names and email
@@ -238,6 +239,11 @@ include::
Name of a configfile to include before the rest of the current config-
file is parsed. Default value: none. See also: "MACRO EXPANSION".
+js::
+ Url which specifies the javascript script document to include in all cgit
+ pages. Default value: "/cgit.js". Setting this to an empty string will
+ disable generation of the link to this file in the head section.
+
local-time::
Flag which, if set to "1", makes cgit print commit and tag times in the
servers timezone. Default value: "0".
@@ -269,7 +275,8 @@ max-message-length::
max-repo-count::
Specifies the number of entries to list per page on the repository
- index page. Default value: "50".
+ index page. The value "0" shows all repositories without limitation.
+ Default value: "50".
max-repodesc-length::
Specifies the maximum number of repo description characters to display
@@ -407,9 +414,12 @@ side-by-side-diffs::
snapshots::
Text which specifies the default set of snapshot formats that cgit
generates links for. The value is a space-separated list of zero or
- more of the values "tar", "tar.gz", "tar.bz2", "tar.xz" and "zip".
- The special value "all" enables all snapshot formats.
- Default value: none.
+ more of the values "tar", "tar.gz", "tar.bz2", "tar.lz", "tar.xz",
+ "tar.zst" and "zip". The special value "all" enables all snapshot
+ formats. Default value: none.
+ All compressors use default settings. Some settings can be influenced
+ with environment variables, for example set ZSTD_CLEVEL=10 in web
+ server environment for higher (but slower) zstd compression.
source-filter::
Specifies a command which will be invoked to format plaintext blobs
@@ -576,11 +586,11 @@ repo.readme::
verbatim as the "About" page for this repo. You may also specify a
git refspec by head or by hash by prepending the refspec followed by
a colon. For example, "master:docs/readme.mkd". If the value begins
- with a colon, i.e. ":docs/readme.rst", the default branch of the
- repository will be used. Sharing any file will expose that entire
- directory tree to the "/about/PATH" endpoints, so be sure that there
- are no non-public files located in the same directory as the readme
- file. Default value: <readme>.
+ with a colon, i.e. ":docs/readme.rst", the head giving in query or
+ the default branch of the repository will be used. Sharing any file
+ will expose that entire directory tree to the "/about/PATH" endpoints,
+ so be sure that there are no non-public files located in the same
+ directory as the readme file. Default value: <readme>.
repo.section::
Override the current section name for this repository. Default value:
diff --git a/cmd.c b/cmd.c
index bf6d8f5..0eb75b1 100644
--- a/cmd.c
+++ b/cmd.c
@@ -74,22 +74,22 @@ static void blame_fn(void)
static void blob_fn(void)
{
- cgit_print_blob(ctx.qry.sha1, ctx.qry.path, ctx.qry.head, 0);
+ cgit_print_blob(ctx.qry.oid, ctx.qry.path, ctx.qry.head, 0);
}
static void commit_fn(void)
{
- cgit_print_commit(ctx.qry.sha1, ctx.qry.path);
+ cgit_print_commit(ctx.qry.oid, ctx.qry.path);
}
static void diff_fn(void)
{
- cgit_print_diff(ctx.qry.sha1, ctx.qry.sha2, ctx.qry.path, 1, 0);
+ cgit_print_diff(ctx.qry.oid, ctx.qry.oid2, ctx.qry.path, 1, 0);
}
static void rawdiff_fn(void)
{
- cgit_print_diff(ctx.qry.sha1, ctx.qry.sha2, ctx.qry.path, 1, 1);
+ cgit_print_diff(ctx.qry.oid, ctx.qry.oid2, ctx.qry.path, 1, 1);
}
static void info_fn(void)
@@ -99,7 +99,7 @@ static void info_fn(void)
static void log_fn(void)
{
- cgit_print_log(ctx.qry.sha1, ctx.qry.ofs, ctx.cfg.max_commit_count,
+ cgit_print_log(ctx.qry.oid, ctx.qry.ofs, ctx.cfg.max_commit_count,
ctx.qry.grep, ctx.qry.search, ctx.qry.path, 1,
ctx.repo->enable_commit_graph,
ctx.repo->commit_sort);
@@ -125,7 +125,7 @@ static void repolist_fn(void)
static void patch_fn(void)
{
- cgit_print_patch(ctx.qry.sha1, ctx.qry.sha2, ctx.qry.path);
+ cgit_print_patch(ctx.qry.oid, ctx.qry.oid2, ctx.qry.path);
}
static void plain_fn(void)
@@ -140,7 +140,7 @@ static void refs_fn(void)
static void snapshot_fn(void)
{
- cgit_print_snapshot(ctx.qry.head, ctx.qry.sha1, ctx.qry.path,
+ cgit_print_snapshot(ctx.qry.head, ctx.qry.oid, ctx.qry.path,
ctx.qry.nohead);
}
@@ -156,12 +156,12 @@ static void summary_fn(void)
static void tag_fn(void)
{
- cgit_print_tag(ctx.qry.sha1);
+ cgit_print_tag(ctx.qry.oid);
}
static void tree_fn(void)
{
- cgit_print_tree(ctx.qry.sha1, ctx.qry.path);
+ cgit_print_tree(ctx.qry.oid, ctx.qry.path);
}
#define def_cmd(name, want_repo, want_vpath, is_clone) \
diff --git a/filters/commit-links.sh b/filters/commit-links.sh
index 5881952..796ac30 100755
--- a/filters/commit-links.sh
+++ b/filters/commit-links.sh
@@ -19,7 +19,7 @@ regex=''
# This expression generates links to commits referenced by their SHA1.
regex=$regex'
-s|\b([0-9a-fA-F]{7,40})\b|<a href="./?id=\1">\1</a>|g'
+s|\b([0-9a-fA-F]{7,64})\b|<a href="./?id=\1">\1</a>|g'
# This expression generates links to a fictional bugtracker.
regex=$regex'
diff --git a/filters/html-converters/md2html b/filters/html-converters/md2html
index dc20f42..59f43a8 100755
--- a/filters/html-converters/md2html
+++ b/filters/html-converters/md2html
@@ -86,11 +86,7 @@ div#cgit .markdown-body h1 a.toclink, div#cgit .markdown-body h2 a.toclink, div#
margin: 15px 0;
}
.markdown-body hr {
- background: transparent url("/dirty-shade.png") repeat-x 0 0;
- border: 0 none;
- color: #ccc;
- height: 4px;
- padding: 0;
+ border: 2px solid #ccc;
}
.markdown-body>h2:first-child, .markdown-body>h1:first-child, .markdown-body>h1:first-child+h2, .markdown-body>h3:first-child, .markdown-body>h4:first-child, .markdown-body>h5:first-child, .markdown-body>h6:first-child {
margin-top: 0;
@@ -301,6 +297,7 @@ markdown.markdownFromFile(
"markdown.extensions.fenced_code",
"markdown.extensions.codehilite",
"markdown.extensions.tables",
+ "markdown.extensions.sane_lists",
TocExtension(anchorlink=True)],
extension_configs={
"markdown.extensions.codehilite":{"css_class":"highlight"}})
diff --git a/git b/git
-Subproject d0654dc308b0ba76dd8ed7bbb33c8d8f7aacd78
+Subproject c48035d29b4e524aed3a32f0403676f0d912886
diff --git a/html.c b/html.c
index 7f81965..0bac34b 100644
--- a/html.c
+++ b/html.c
@@ -59,7 +59,7 @@ char *fmt(const char *format, ...)
va_start(args, format);
len = vsnprintf(buf[bufidx], sizeof(buf[bufidx]), format, args);
va_end(args);
- if (len > sizeof(buf[bufidx])) {
+ if (len >= sizeof(buf[bufidx])) {
fprintf(stderr, "[html.c] string truncated: %s\n", format);
exit(1);
}
diff --git a/parsing.c b/parsing.c
index 93b4767..72b59b3 100644
--- a/parsing.c
+++ b/parsing.c
@@ -127,7 +127,6 @@ static int end_of_header(const char *p)
struct commitinfo *cgit_parse_commit(struct commit *commit)
{
- const int sha1hex_len = 40;
struct commitinfo *ret;
const char *p = repo_get_commit_buffer(the_repository, commit, NULL);
const char *t;
@@ -140,10 +139,10 @@ struct commitinfo *cgit_parse_commit(struct commit *commit)
if (!skip_prefix(p, "tree ", &p))
die("Bad commit: %s", oid_to_hex(&commit->object.oid));
- p += sha1hex_len + 1;
+ p += the_hash_algo->hexsz + 1;
while (skip_prefix(p, "parent ", &p))
- p += sha1hex_len + 1;
+ p += the_hash_algo->hexsz + 1;
if (p && skip_prefix(p, "author ", &p)) {
parse_user(p, &ret->author, &ret->author_email,
diff --git a/robots.txt b/robots.txt
index 4ce948f..1b33266 100644
--- a/robots.txt
+++ b/robots.txt
@@ -1,3 +1,4 @@
User-agent: *
Disallow: /*/snapshot/*
+Disallow: /*/blame/*
Allow: /
diff --git a/shared.c b/shared.c
index 8115469..0bceb98 100644
--- a/shared.c
+++ b/shared.c
@@ -341,9 +341,8 @@ void cgit_diff_tree(const struct object_id *old_oid,
filepair_fn fn, const char *prefix, int ignorews)
{
struct diff_options opt;
- struct pathspec_item item;
+ struct pathspec_item *item;
- memset(&item, 0, sizeof(item));
diff_setup(&opt);
opt.output_format = DIFF_FORMAT_CALLBACK;
opt.detect_rename = 1;
@@ -354,10 +353,11 @@ void cgit_diff_tree(const struct object_id *old_oid,
opt.format_callback = cgit_diff_tree_cb;
opt.format_callback_data = fn;
if (prefix) {
- item.match = xstrdup(prefix);
- item.len = strlen(prefix);
+ item = xcalloc(1, sizeof(*item));
+ item->match = xstrdup(prefix);
+ item->len = strlen(prefix);
opt.pathspec.nr = 1;
- opt.pathspec.items = &item;
+ opt.pathspec.items = item;
}
diff_setup_done(&opt);
@@ -367,8 +367,6 @@ void cgit_diff_tree(const struct object_id *old_oid,
diff_root_tree_oid(new_oid, "", &opt);
diffcore_std(&opt);
diff_flush(&opt);
-
- free(item.match);
}
void cgit_diff_commit(struct commit *commit, filepair_fn fn, const char *prefix)
diff --git a/tests/setup.sh b/tests/setup.sh
index 7590f04..8db810f 100755
--- a/tests/setup.sh
+++ b/tests/setup.sh
@@ -80,13 +80,17 @@ mkrepo() {
git commit -m "commit $n"
n=$(expr $n + 1)
done
- if test "$3" = "testplus"
- then
+ case "$3" in
+ testplus)
echo "hello" >a+b
git add a+b
git commit -m "add a+b"
git branch "1+2"
- fi
+ ;;
+ commit-graph)
+ git commit-graph write
+ ;;
+ esac
)
}
@@ -95,7 +99,7 @@ setup_repos()
rm -rf cache
mkdir -p cache
mkrepo repos/foo 5 >/dev/null
- mkrepo repos/bar 50 >/dev/null
+ mkrepo repos/bar 50 commit-graph >/dev/null
mkrepo repos/foo+bar 10 testplus >/dev/null
mkrepo "repos/with space" 2 >/dev/null
mkrepo repos/filter 5 testplus >/dev/null
@@ -104,7 +108,7 @@ virtual-root=/
cache-root=$PWD/cache
cache-size=1021
-snapshots=tar.gz tar.bz zip
+snapshots=tar.gz tar.bz tar.lz tar.xz tar.zst zip
enable-log-filecount=1
enable-log-linecount=1
summary-log=5
diff --git a/tests/t0001-validate-git-versions.sh b/tests/t0001-validate-git-versions.sh
index 3200f31..dd84fe3 100755
--- a/tests/t0001-validate-git-versions.sh
+++ b/tests/t0001-validate-git-versions.sh
@@ -1,5 +1,9 @@
#!/bin/sh
+if [ "${CGIT_TEST_NO_GIT_VERSION}" = "YesPlease" ]; then
+ exit 0
+fi
+
test_description='Check Git version is correct'
CGIT_TEST_NO_CREATE_REPOS=YesPlease
. ./setup.sh
@@ -29,10 +33,10 @@ test_expect_success 'test submodule version matches Makefile' '
else
(
cd ../.. &&
- sm_sha1=$(git ls-files --stage -- git |
+ sm_oid=$(git ls-files --stage -- git |
sed -e "s/^[0-9]* \\([0-9a-f]*\\) [0-9] .*$/\\1/") &&
cd git &&
- git describe --match "v[0-9]*" $sm_sha1
+ git describe --match "v[0-9]*" $sm_oid
) | sed -e "s/^v//" -e "s/-/./" >sm_version &&
test_cmp sm_version makefile_version
fi
diff --git a/tests/t0105-commit.sh b/tests/t0105-commit.sh
index 9cdf55c..1a12ee3 100755
--- a/tests/t0105-commit.sh
+++ b/tests/t0105-commit.sh
@@ -25,7 +25,7 @@ test_expect_success 'get root commit' '
'
test_expect_success 'root commit contains diffstat' '
- grep "<a href=./foo/diff/file-1.id=[0-9a-f]\{40\}.>file-1</a>" tmp
+ grep "<a href=./foo/diff/file-1.id=[0-9a-f]\{40,64\}.>file-1</a>" tmp
'
test_expect_success 'root commit contains diff' '
diff --git a/tests/t0107-snapshot.sh b/tests/t0107-snapshot.sh
index 6cf7aaa..0811ec4 100755
--- a/tests/t0107-snapshot.sh
+++ b/tests/t0107-snapshot.sh
@@ -25,7 +25,7 @@ test_expect_success 'verify gzip format' '
test_expect_success 'untar' '
rm -rf master &&
- tar -xzf master.tar.gz
+ gzip -dc master.tar.gz | tar -xf -
'
test_expect_success 'count files' '
@@ -38,6 +38,129 @@ test_expect_success 'verify untarred file-5' '
test_line_count = 1 master/file-5
'
+if test -n "$(which lzip 2>/dev/null)"; then
+ test_set_prereq LZIP
+else
+ say 'Skipping LZIP validation tests: lzip not found'
+fi
+
+test_expect_success LZIP 'get foo/snapshot/master.tar.lz' '
+ cgit_url "foo/snapshot/master.tar.lz" >tmp
+'
+
+test_expect_success LZIP 'check html headers' '
+ head -n 1 tmp |
+ grep "Content-Type: application/x-lzip" &&
+
+ head -n 2 tmp |
+ grep "Content-Disposition: inline; filename=.master.tar.lz."
+'
+
+test_expect_success LZIP 'strip off the header lines' '
+ strip_headers <tmp >master.tar.lz
+'
+
+test_expect_success LZIP 'verify lzip format' '
+ lzip --test master.tar.lz
+'
+
+test_expect_success LZIP 'untar' '
+ rm -rf master &&
+ lzip -dc master.tar.lz | tar -xf -
+'
+
+test_expect_success LZIP 'count files' '
+ ls master/ >output &&
+ test_line_count = 5 output
+'
+
+test_expect_success LZIP 'verify untarred file-5' '
+ grep "^5$" master/file-5 &&
+ test_line_count = 1 master/file-5
+'
+
+if test -n "$(which xz 2>/dev/null)"; then
+ test_set_prereq XZ
+else
+ say 'Skipping XZ validation tests: xz not found'
+fi
+
+test_expect_success XZ 'get foo/snapshot/master.tar.xz' '
+ cgit_url "foo/snapshot/master.tar.xz" >tmp
+'
+
+test_expect_success XZ 'check html headers' '
+ head -n 1 tmp |
+ grep "Content-Type: application/x-xz" &&
+
+ head -n 2 tmp |
+ grep "Content-Disposition: inline; filename=.master.tar.xz."
+'
+
+test_expect_success XZ 'strip off the header lines' '
+ strip_headers <tmp >master.tar.xz
+'
+
+test_expect_success XZ 'verify xz format' '
+ xz --test master.tar.xz
+'
+
+test_expect_success XZ 'untar' '
+ rm -rf master &&
+ xz -dc master.tar.xz | tar -xf -
+'
+
+test_expect_success XZ 'count files' '
+ ls master/ >output &&
+ test_line_count = 5 output
+'
+
+test_expect_success XZ 'verify untarred file-5' '
+ grep "^5$" master/file-5 &&
+ test_line_count = 1 master/file-5
+'
+
+if test -n "$(which zstd 2>/dev/null)"; then
+ test_set_prereq ZSTD
+else
+ say 'Skipping ZSTD validation tests: zstd not found'
+fi
+
+test_expect_success ZSTD 'get foo/snapshot/master.tar.zst' '
+ cgit_url "foo/snapshot/master.tar.zst" >tmp
+'
+
+test_expect_success ZSTD 'check html headers' '
+ head -n 1 tmp |
+ grep "Content-Type: application/x-zstd" &&
+
+ head -n 2 tmp |
+ grep "Content-Disposition: inline; filename=.master.tar.zst."
+'
+
+test_expect_success ZSTD 'strip off the header lines' '
+ strip_headers <tmp >master.tar.zst
+'
+
+test_expect_success ZSTD 'verify zstd format' '
+ zstd --test master.tar.zst
+'
+
+test_expect_success ZSTD 'untar' '
+ rm -rf master &&
+ zstd -dc master.tar.zst | tar -xf -
+'
+
+test_expect_success ZSTD 'count files' '
+ ls master/ >output &&
+ test_line_count = 5 output
+'
+
+test_expect_success ZSTD 'verify untarred file-5' '
+ grep "^5$" master/file-5 &&
+ test_line_count = 1 master/file-5
+'
+
test_expect_success 'get foo/snapshot/master.zip' '
cgit_url "foo/snapshot/master.zip" >tmp
'
diff --git a/tests/t0109-gitconfig.sh b/tests/t0109-gitconfig.sh
index 8cee75c..189ef28 100755
--- a/tests/t0109-gitconfig.sh
+++ b/tests/t0109-gitconfig.sh
@@ -25,7 +25,7 @@ test_no_home_access () {
-E CGIT_CONFIG="$PWD/cgitrc" \
-E QUERY_STRING="url=$1" \
-e access -f -o strace.out cgit &&
- test_must_fail grep "$non_existent_path" strace.out
+ ! grep "$non_existent_path" strace.out
}
test_no_home_access_success() {
diff --git a/ui-atom.c b/ui-atom.c
index 1056f36..5f4ad7d 100644
--- a/ui-atom.c
+++ b/ui-atom.c
@@ -67,17 +67,12 @@ static void add_entry(struct commit *commit, const char *host)
html("'/>\n");
free(pageurl);
}
- htmlf("<id>%s</id>\n", hex);
+ html("<id>");
+ html_txtf("urn:%s:%s", the_hash_algo->name, hex);
+ html("</id>\n");
html("<content type='text'>\n");
html_txt(info->msg);
html("</content>\n");
- html("<content type='xhtml'>\n");
- html("<div xmlns='http://www.w3.org/1999/xhtml'>\n");
- html("<pre>\n");
- html_txt(info->msg);
- html("</pre>\n");
- html("</div>\n");
- html("</content>\n");
html("</entry>\n");
cgit_free_commitinfo(info);
}
@@ -90,6 +85,7 @@ void cgit_print_atom(char *tip, const char *path, int max_count)
struct commit *commit;
struct rev_info rev;
int argc = 2;
+ bool first = true;
if (ctx.qry.show_all)
argv[1] = "--all";
@@ -130,18 +126,30 @@ void cgit_print_atom(char *tip, const char *path, int max_count)
html_txt(ctx.repo->desc);
html("</subtitle>\n");
if (host) {
+ char *fullurl = cgit_currentfullurl();
char *repourl = cgit_repourl(ctx.repo->url);
+ html("<id>");
+ html_txtf("%s%s%s", cgit_httpscheme(), host, fullurl);
+ html("</id>\n");
+ html("<link rel='self' href='");
+ html_attrf("%s%s%s", cgit_httpscheme(), host, fullurl);
+ html("'/>\n");
html("<link rel='alternate' type='text/html' href='");
- html(cgit_httpscheme());
- html_attr(host);
- html_attr(repourl);
+ html_attrf("%s%s%s", cgit_httpscheme(), host, repourl);
html("'/>\n");
+ free(fullurl);
free(repourl);
}
while ((commit = get_revision(&rev)) != NULL) {
+ if (first) {
+ html("<updated>");
+ html_txt(show_date(commit->date, 0,
+ date_mode_from_type(DATE_ISO8601_STRICT)));
+ html("</updated>\n");
+ first = false;
+ }
add_entry(commit, host);
- free_commit_buffer(the_repository->parsed_objects, commit);
- free_commit_list(commit->parents);
+ release_commit_memory(the_repository->parsed_objects, commit);
commit->parents = NULL;
}
html("</feed>\n");
diff --git a/ui-blame.c b/ui-blame.c
index 644c30a..aedce8d 100644
--- a/ui-blame.c
+++ b/ui-blame.c
@@ -10,7 +10,7 @@
#include "ui-blame.h"
#include "html.h"
#include "ui-shared.h"
-#include "argv-array.h"
+#include "strvec.h"
#include "blame.h"
@@ -48,12 +48,21 @@ static void emit_blame_entry_hash(struct blame_entry *ent)
unsigned long line = 0;
char *detail = emit_suspect_detail(suspect);
- html("<span class='sha1'>");
+ html("<span class='oid'>");
cgit_commit_link(find_unique_abbrev(oid, DEFAULT_ABBREV), detail,
NULL, ctx.qry.head, oid_to_hex(oid), suspect->path);
html("</span>");
free(detail);
+ if (!parse_commit(suspect->commit) && suspect->commit->parents) {
+ struct commit *parent = suspect->commit->parents->item;
+
+ html(" ");
+ cgit_blame_link("^", "Blame the previous revision", NULL,
+ ctx.qry.head, oid_to_hex(&parent->object.oid),
+ suspect->path);
+ }
+
while (line++ < ent->num_lines)
html("\n");
}
@@ -104,7 +113,7 @@ static void print_object(const struct object_id *oid, const char *path,
enum object_type type;
char *buf;
unsigned long size;
- struct argv_array rev_argv = ARGV_ARRAY_INIT;
+ struct strvec rev_argv = STRVEC_INIT;
struct rev_info revs;
struct blame_scoreboard sb;
struct blame_origin *o;
@@ -124,15 +133,16 @@ static void print_object(const struct object_id *oid, const char *path,
return;
}
- argv_array_push(&rev_argv, "blame");
- argv_array_push(&rev_argv, rev);
+ strvec_push(&rev_argv, "blame");
+ strvec_push(&rev_argv, rev);
init_revisions(&revs, NULL);
revs.diffopt.flags.allow_textconv = 1;
- setup_revisions(rev_argv.argc, rev_argv.argv, &revs, NULL);
+ setup_revisions(rev_argv.nr, rev_argv.v, &revs, NULL);
init_scoreboard(&sb);
sb.revs = &revs;
sb.repo = the_repository;
- setup_scoreboard(&sb, path, &o);
+ sb.path = path;
+ setup_scoreboard(&sb, &o);
o->suspects = blame_entry_prepend(NULL, 0, sb.num_lines, o);
prio_queue_put(&sb.commits, o->commit);
blame_origin_decref(o);
@@ -151,6 +161,10 @@ static void print_object(const struct object_id *oid, const char *path,
cgit_tree_link("tree", NULL, NULL, ctx.qry.head, rev, path);
html(")\n");
+ if (buffer_is_binary(buf, size)) {
+ html("<div class='error'>blob is binary.</div>");
+ goto cleanup;
+ }
if (ctx.cfg.max_blob_size && size / 1024 > ctx.cfg.max_blob_size) {
htmlf("<div class='error'>blob size (%ldKB)"
" exceeds display size limit (%dKB).</div>",
@@ -220,8 +234,7 @@ cleanup:
}
static int walk_tree(const struct object_id *oid, struct strbuf *base,
- const char *pathname, unsigned mode, int stage,
- void *cbdata)
+ const char *pathname, unsigned mode, void *cbdata)
{
struct walk_tree_context *walk_tree_ctx = cbdata;
@@ -256,7 +269,7 @@ static int basedir_len(const char *path)
void cgit_print_blame(void)
{
- const char *rev = ctx.qry.sha1;
+ const char *rev = ctx.qry.oid;
struct object_id oid;
struct commit *commit;
struct pathspec_item path_items = {
@@ -290,8 +303,8 @@ void cgit_print_blame(void)
walk_tree_ctx.match_baselen = (path_items.match) ?
basedir_len(path_items.match) : -1;
- read_tree_recursive(the_repository, commit->maybe_tree, "", 0, 0,
- &paths, walk_tree, &walk_tree_ctx);
+ read_tree(the_repository, repo_get_commit_tree(the_repository, commit),
+ &paths, walk_tree, &walk_tree_ctx);
if (!walk_tree_ctx.state)
cgit_print_error_page(404, "Not found", "Not found");
else if (walk_tree_ctx.state == 2)
diff --git a/ui-blob.c b/ui-blob.c
index 30e2d4b..c10ae42 100644
--- a/ui-blob.c
+++ b/ui-blob.c
@@ -19,7 +19,7 @@ struct walk_tree_context {
};
static int walk_tree(const struct object_id *oid, struct strbuf *base,
- const char *pathname, unsigned mode, int stage, void *cbdata)
+ const char *pathname, unsigned mode, void *cbdata)
{
struct walk_tree_context *walk_tree_ctx = cbdata;
@@ -56,8 +56,9 @@ int cgit_ref_path_exists(const char *path, const char *ref, int file_only)
goto done;
if (oid_object_info(the_repository, &oid, &size) != OBJ_COMMIT)
goto done;
- read_tree_recursive(the_repository, lookup_commit_reference(the_repository, &oid)->maybe_tree,
- "", 0, 0, &paths, walk_tree, &walk_tree_ctx);
+ read_tree(the_repository,
+ repo_get_commit_tree(the_repository, lookup_commit_reference(the_repository, &oid)),
+ &paths, walk_tree, &walk_tree_ctx);
done:
free(path_items.match);
@@ -91,8 +92,8 @@ int cgit_print_file(char *path, const char *head, int file_only)
type = oid_object_info(the_repository, &oid, &size);
if (type == OBJ_COMMIT) {
commit = lookup_commit_reference(the_repository, &oid);
- read_tree_recursive(the_repository, commit->maybe_tree,
- "", 0, 0, &paths, walk_tree, &walk_tree_ctx);
+ read_tree(the_repository, repo_get_commit_tree(the_repository, commit),
+ &paths, walk_tree, &walk_tree_ctx);
if (!walk_tree_ctx.found_path)
return -1;
type = oid_object_info(the_repository, &oid, &size);
@@ -148,8 +149,8 @@ void cgit_print_blob(const char *hex, char *path, const char *head, int file_onl
if ((!hex) && type == OBJ_COMMIT && path) {
commit = lookup_commit_reference(the_repository, &oid);
- read_tree_recursive(the_repository, commit->maybe_tree,
- "", 0, 0, &paths, walk_tree, &walk_tree_ctx);
+ read_tree(the_repository, repo_get_commit_tree(the_repository, commit),
+ &paths, walk_tree, &walk_tree_ctx);
type = oid_object_info(the_repository, &oid, &size);
}
diff --git a/ui-commit.c b/ui-commit.c
index 9a47b54..0787237 100644
--- a/ui-commit.c
+++ b/ui-commit.c
@@ -39,10 +39,11 @@ void cgit_print_commit(char *hex, const char *prefix)
}
info = cgit_parse_commit(commit);
- format_display_notes(&oid, &notes, PAGE_ENCODING, 0);
+ format_display_notes(&oid, &notes, PAGE_ENCODING, 1);
load_ref_decorations(NULL, DECORATE_FULL_REFS);
+ ctx.page.title = fmtalloc("%s - %s", info->subject, ctx.page.title);
cgit_print_layout_start();
cgit_print_diff_ctrls();
html("<table summary='commit info' class='commit-info'>\n");
@@ -70,15 +71,15 @@ void cgit_print_commit(char *hex, const char *prefix)
html_txt(show_date(info->committer_date, info->committer_tz,
cgit_date_mode(DATE_ISO8601)));
html("</td></tr>\n");
- html("<tr><th>commit</th><td colspan='2' class='sha1'>");
+ html("<tr><th>commit</th><td colspan='2' class='oid'>");
tmp = oid_to_hex(&commit->object.oid);
cgit_commit_link(tmp, NULL, NULL, ctx.qry.head, tmp, prefix);
html(" (");
cgit_patch_link("patch", NULL, NULL, NULL, tmp, prefix);
html(")</td></tr>\n");
- html("<tr><th>tree</th><td colspan='2' class='sha1'>");
+ html("<tr><th>tree</th><td colspan='2' class='oid'>");
tmp = xstrdup(hex);
- cgit_tree_link(oid_to_hex(&commit->maybe_tree->object.oid), NULL, NULL,
+ cgit_tree_link(oid_to_hex(get_commit_tree_oid(commit)), NULL, NULL,
ctx.qry.head, tmp, NULL);
if (prefix) {
html(" /");
@@ -95,7 +96,7 @@ void cgit_print_commit(char *hex, const char *prefix)
continue;
}
html("<tr><th>parent</th>"
- "<td colspan='2' class='sha1'>");
+ "<td colspan='2' class='oid'>");
tmp = tmp2 = oid_to_hex(&p->item->object.oid);
if (ctx.repo->enable_subject_links) {
parent_info = cgit_parse_commit(parent);
@@ -109,7 +110,7 @@ void cgit_print_commit(char *hex, const char *prefix)
parents++;
}
if (ctx.repo->snapshots) {
- html("<tr><th>download</th><td colspan='2' class='sha1'>");
+ html("<tr><th>download</th><td colspan='2' class='oid'>");
cgit_print_snapshot_links(ctx.repo, hex, "<br/>");
html("</td></tr>");
}
@@ -139,7 +140,7 @@ void cgit_print_commit(char *hex, const char *prefix)
tmp = oid_to_hex(&commit->parents->item->object.oid);
else
tmp = NULL;
- cgit_print_diff(ctx.qry.sha1, tmp, prefix, 0, 0);
+ cgit_print_diff(ctx.qry.oid, tmp, prefix, 0, 0);
}
strbuf_release(&notes);
cgit_free_commitinfo(info);
diff --git a/ui-diff.c b/ui-diff.c
index c60aefd..5ed5990 100644
--- a/ui-diff.c
+++ b/ui-diff.c
@@ -97,8 +97,8 @@ static void print_fileinfo(struct fileinfo *info)
html("]</span>");
}
htmlf("</td><td class='%s'>", class);
- cgit_diff_link(info->new_path, NULL, NULL, ctx.qry.head, ctx.qry.sha1,
- ctx.qry.sha2, info->new_path);
+ cgit_diff_link(info->new_path, NULL, NULL, ctx.qry.head, ctx.qry.oid,
+ ctx.qry.oid2, info->new_path);
if (info->status == DIFF_STATUS_COPIED || info->status == DIFF_STATUS_RENAMED) {
htmlf(" (%s from ",
info->status == DIFF_STATUS_COPIED ? "copied" : "renamed");
@@ -194,8 +194,8 @@ static void cgit_print_diffstat(const struct object_id *old_oid,
int i;
html("<div class='diffstat-header'>");
- cgit_diff_link("Diffstat", NULL, NULL, ctx.qry.head, ctx.qry.sha1,
- ctx.qry.sha2, NULL);
+ cgit_diff_link("Diffstat", NULL, NULL, ctx.qry.head, ctx.qry.oid,
+ ctx.qry.oid2, NULL);
if (prefix) {
html(" (limited to '");
html_txt(prefix);
@@ -413,7 +413,7 @@ void cgit_print_diff(const char *new_rev, const char *old_rev,
"Bad commit: %s", oid_to_hex(new_rev_oid));
return;
}
- new_tree_oid = &commit->maybe_tree->object.oid;
+ new_tree_oid = get_commit_tree_oid(commit);
if (old_rev) {
if (get_oid(old_rev, old_rev_oid)) {
@@ -434,7 +434,7 @@ void cgit_print_diff(const char *new_rev, const char *old_rev,
"Bad commit: %s", oid_to_hex(old_rev_oid));
return;
}
- old_tree_oid = &commit2->maybe_tree->object.oid;
+ old_tree_oid = get_commit_tree_oid(commit2);
} else {
old_tree_oid = NULL;
}
diff --git a/ui-log.c b/ui-log.c
index dc5cb1e..311304a 100644
--- a/ui-log.c
+++ b/ui-log.c
@@ -10,7 +10,7 @@
#include "ui-log.h"
#include "html.h"
#include "ui-shared.h"
-#include "argv-array.h"
+#include "strvec.h"
static int files, add_lines, rem_lines, lines_counted;
@@ -65,8 +65,9 @@ void show_commit_decorations(struct commit *commit)
return;
html("<span class='decoration'>");
while (deco) {
- struct object_id peeled;
+ struct object_id oid_tag, peeled;
int is_annotated = 0;
+
strlcpy(buf, prettify_refname(deco->name), sizeof(buf));
switch(deco->type) {
case DECORATION_NONE:
@@ -79,8 +80,8 @@ void show_commit_decorations(struct commit *commit)
ctx.qry.showmsg, 0);
break;
case DECORATION_REF_TAG:
- if (!peel_ref(deco->name, &peeled))
- is_annotated = !oidcmp(&commit->object.oid, &peeled);
+ if (!read_ref(deco->name, &oid_tag) && !peel_iterated_oid(&oid_tag, &peeled))
+ is_annotated = !oideq(&oid_tag, &peeled);
cgit_tag_link(buf, NULL, is_annotated ? "tag-annotated-deco" : "tag-deco", buf);
break;
case DECORATION_REF_REMOTE:
@@ -153,12 +154,12 @@ static int show_commit(struct commit *commit, struct rev_info *revs)
rem_lines = 0;
revs->diffopt.flags.recursive = 1;
- diff_tree_oid(&parent->maybe_tree->object.oid,
- &commit->maybe_tree->object.oid,
+ diff_tree_oid(get_commit_tree_oid(parent),
+ get_commit_tree_oid(commit),
"", &revs->diffopt);
diffcore_std(&revs->diffopt);
- found = !diff_queue_is_empty();
+ found = !diff_queue_is_empty(&revs->diffopt);
saved_fmt = revs->diffopt.output_format;
revs->diffopt.output_format = DIFF_FORMAT_CALLBACK;
revs->diffopt.format_callback = cgit_diff_tree_cb;
@@ -366,23 +367,23 @@ void cgit_print_log(const char *tip, int ofs, int cnt, char *grep, char *pattern
{
struct rev_info rev;
struct commit *commit;
- struct argv_array rev_argv = ARGV_ARRAY_INIT;
+ struct strvec rev_argv = STRVEC_INIT;
int i, columns = commit_graph ? 4 : 3;
int must_free_tip = 0;
/* rev_argv.argv[0] will be ignored by setup_revisions */
- argv_array_push(&rev_argv, "log_rev_setup");
+ strvec_push(&rev_argv, "log_rev_setup");
if (!tip)
tip = ctx.qry.head;
tip = disambiguate_ref(tip, &must_free_tip);
- argv_array_push(&rev_argv, tip);
+ strvec_push(&rev_argv, tip);
if (grep && pattern && *pattern) {
pattern = xstrdup(pattern);
if (!strcmp(grep, "grep") || !strcmp(grep, "author") ||
!strcmp(grep, "committer")) {
- argv_array_pushf(&rev_argv, "--%s=%s", grep, pattern);
+ strvec_pushf(&rev_argv, "--%s=%s", grep, pattern);
} else if (!strcmp(grep, "range")) {
char *arg;
/* Split the pattern at whitespace and add each token
@@ -390,14 +391,14 @@ void cgit_print_log(const char *tip, int ofs, int cnt, char *grep, char *pattern
* rev-list options. Also, replace the previously
* pushed tip (it's no longer relevant).
*/
- argv_array_pop(&rev_argv);
+ strvec_pop(&rev_argv);
while ((arg = next_token(&pattern))) {
if (*arg == '-') {
fprintf(stderr, "Bad range expr: %s\n",
arg);
break;
}
- argv_array_push(&rev_argv, arg);
+ strvec_push(&rev_argv, arg);
}
}
}
@@ -412,22 +413,22 @@ void cgit_print_log(const char *tip, int ofs, int cnt, char *grep, char *pattern
}
if (commit_graph && !ctx.qry.follow) {
- argv_array_push(&rev_argv, "--graph");
- argv_array_push(&rev_argv, "--color");
+ strvec_push(&rev_argv, "--graph");
+ strvec_push(&rev_argv, "--color");
graph_set_column_colors(column_colors_html,
COLUMN_COLORS_HTML_MAX);
}
if (commit_sort == 1)
- argv_array_push(&rev_argv, "--date-order");
+ strvec_push(&rev_argv, "--date-order");
else if (commit_sort == 2)
- argv_array_push(&rev_argv, "--topo-order");
+ strvec_push(&rev_argv, "--topo-order");
if (path && ctx.qry.follow)
- argv_array_push(&rev_argv, "--follow");
- argv_array_push(&rev_argv, "--");
+ strvec_push(&rev_argv, "--follow");
+ strvec_push(&rev_argv, "--");
if (path)
- argv_array_push(&rev_argv, path);
+ strvec_push(&rev_argv, path);
init_revisions(&rev, NULL);
rev.abbrev = DEFAULT_ABBREV;
@@ -436,7 +437,7 @@ void cgit_print_log(const char *tip, int ofs, int cnt, char *grep, char *pattern
rev.show_root_diff = 0;
rev.ignore_missing = 1;
rev.simplify_history = 1;
- setup_revisions(rev_argv.argc, rev_argv.argv, &rev, NULL);
+ setup_revisions(rev_argv.nr, rev_argv.v, &rev, NULL);
load_ref_decorations(NULL, DECORATE_FULL_REFS);
rev.show_decorations = 1;
rev.grep_filter.ignore_case = 1;
@@ -463,7 +464,7 @@ void cgit_print_log(const char *tip, int ofs, int cnt, char *grep, char *pattern
if (pager) {
html(" (");
cgit_log_link(ctx.qry.showmsg ? "Collapse" : "Expand", NULL,
- NULL, ctx.qry.head, ctx.qry.sha1,
+ NULL, ctx.qry.head, ctx.qry.oid,
ctx.qry.vpath, ctx.qry.ofs, ctx.qry.grep,
ctx.qry.search, ctx.qry.showmsg ? 0 : 1,
ctx.qry.follow);
@@ -488,8 +489,7 @@ void cgit_print_log(const char *tip, int ofs, int cnt, char *grep, char *pattern
for (i = 0; i < ofs && (commit = get_revision(&rev)) != NULL; /* nop */) {
if (show_commit(commit, &rev))
i++;
- free_commit_buffer(the_repository->parsed_objects, commit);
- free_commit_list(commit->parents);
+ release_commit_memory(the_repository->parsed_objects, commit);
commit->parents = NULL;
}
@@ -510,8 +510,7 @@ void cgit_print_log(const char *tip, int ofs, int cnt, char *grep, char *pattern
i++;
print_commit(commit, &rev);
}
- free_commit_buffer(the_repository->parsed_objects, commit);
- free_commit_list(commit->parents);
+ release_commit_memory(the_repository->parsed_objects, commit);
commit->parents = NULL;
}
if (pager) {
@@ -519,7 +518,7 @@ void cgit_print_log(const char *tip, int ofs, int cnt, char *grep, char *pattern
if (ofs > 0) {
html("<li>");
cgit_log_link("[prev]", NULL, NULL, ctx.qry.head,
- ctx.qry.sha1, ctx.qry.vpath,
+ ctx.qry.oid, ctx.qry.vpath,
ofs - cnt, ctx.qry.grep,
ctx.qry.search, ctx.qry.showmsg,
ctx.qry.follow);
@@ -528,7 +527,7 @@ void cgit_print_log(const char *tip, int ofs, int cnt, char *grep, char *pattern
if ((commit = get_revision(&rev)) != NULL) {
html("<li>");
cgit_log_link("[next]", NULL, NULL, ctx.qry.head,
- ctx.qry.sha1, ctx.qry.vpath,
+ ctx.qry.oid, ctx.qry.vpath,
ofs + cnt, ctx.qry.grep,
ctx.qry.search, ctx.qry.showmsg,
ctx.qry.follow);
diff --git a/ui-patch.c b/ui-patch.c
index 5a96410..4ac03cb 100644
--- a/ui-patch.c
+++ b/ui-patch.c
@@ -61,7 +61,7 @@ void cgit_print_patch(const char *new_rev, const char *old_rev,
}
if (is_null_oid(&old_rev_oid)) {
- memcpy(rev_range, oid_to_hex(&new_rev_oid), GIT_SHA1_HEXSZ + 1);
+ memcpy(rev_range, oid_to_hex(&new_rev_oid), the_hash_algo->hexsz + 1);
} else {
xsnprintf(rev_range, REV_RANGE_LEN, "%s..%s", oid_to_hex(&old_rev_oid),
oid_to_hex(&new_rev_oid));
diff --git a/ui-plain.c b/ui-plain.c
index b73c1cf..65a205f 100644
--- a/ui-plain.c
+++ b/ui-plain.c
@@ -99,7 +99,7 @@ static void print_dir(const struct object_id *oid, const char *base,
fullpath = NULL;
}
html("<li>");
- cgit_plain_link("../", NULL, NULL, ctx.qry.head, ctx.qry.sha1,
+ cgit_plain_link("../", NULL, NULL, ctx.qry.head, ctx.qry.oid,
fullpath);
html("</li>\n");
}
@@ -118,7 +118,7 @@ static void print_dir_entry(const struct object_id *oid, const char *base,
if (S_ISGITLINK(mode)) {
cgit_submodule_link(NULL, fullpath, oid_to_hex(oid));
} else
- cgit_plain_link(path, NULL, NULL, ctx.qry.head, ctx.qry.sha1,
+ cgit_plain_link(path, NULL, NULL, ctx.qry.head, ctx.qry.oid,
fullpath);
html("</li>\n");
free(fullpath);
@@ -130,7 +130,7 @@ static void print_dir_tail(void)
}
static int walk_tree(const struct object_id *oid, struct strbuf *base,
- const char *pathname, unsigned mode, int stage, void *cbdata)
+ const char *pathname, unsigned mode, void *cbdata)
{
struct walk_tree_context *walk_tree_ctx = cbdata;
@@ -163,7 +163,7 @@ static int basedir_len(const char *path)
void cgit_print_plain(void)
{
- const char *rev = ctx.qry.sha1;
+ const char *rev = ctx.qry.oid;
struct object_id oid;
struct commit *commit;
struct pathspec_item path_items = {
@@ -193,13 +193,13 @@ void cgit_print_plain(void)
if (!path_items.match) {
path_items.match = "";
walk_tree_ctx.match_baselen = -1;
- print_dir(&commit->maybe_tree->object.oid, "", 0, "");
+ print_dir(get_commit_tree_oid(commit), "", 0, "");
walk_tree_ctx.match = 2;
}
else
walk_tree_ctx.match_baselen = basedir_len(path_items.match);
- read_tree_recursive(the_repository, commit->maybe_tree,
- "", 0, 0, &paths, walk_tree, &walk_tree_ctx);
+ read_tree(the_repository, repo_get_commit_tree(the_repository, commit),
+ &paths, walk_tree, &walk_tree_ctx);
if (!walk_tree_ctx.match)
cgit_print_error_page(404, "Not found", "Not found");
else if (walk_tree_ctx.match == 2)
diff --git a/ui-repolist.c b/ui-repolist.c
index 529a203..d12e3dd 100644
--- a/ui-repolist.c
+++ b/ui-repolist.c
@@ -321,7 +321,7 @@ void cgit_print_repolist(void)
}
htmlf("<tr><td class='%s'>",
!sorted && section ? "sublevel-repo" : "toplevel-repo");
- cgit_summary_link(ctx.repo->name, ctx.repo->name, NULL, NULL);
+ cgit_summary_link(ctx.repo->name, NULL, NULL, NULL);
html("</td><td>");
repourl = cgit_repourl(ctx.repo->url);
html_link_open(repourl, NULL, NULL);
diff --git a/ui-shared.c b/ui-shared.c
index d2358f2..baea6f2 100644
--- a/ui-shared.c
+++ b/ui-shared.c
@@ -22,10 +22,11 @@ static char *http_date(time_t t)
static char month[][4] =
{"Jan", "Feb", "Mar", "Apr", "May", "Jun",
"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
- struct tm *tm = gmtime(&t);
- return fmt("%s, %02d %s %04d %02d:%02d:%02d GMT", day[tm->tm_wday],
- tm->tm_mday, month[tm->tm_mon], 1900 + tm->tm_year,
- tm->tm_hour, tm->tm_min, tm->tm_sec);
+ struct tm tm;
+ gmtime_r(&t, &tm);
+ return fmt("%s, %02d %s %04d %02d:%02d:%02d GMT", day[tm.tm_wday],
+ tm.tm_mday, month[tm.tm_mon], 1900 + tm.tm_year,
+ tm.tm_hour, tm.tm_min, tm.tm_sec);
}
void cgit_print_error(const char *fmt, ...)
@@ -521,45 +522,45 @@ static void cgit_self_link(char *name, const char *title, const char *class)
else if (!strcmp(ctx.qry.page, "summary"))
cgit_summary_link(name, title, class, ctx.qry.head);
else if (!strcmp(ctx.qry.page, "tag"))
- cgit_tag_link(name, title, class, ctx.qry.has_sha1 ?
- ctx.qry.sha1 : ctx.qry.head);
+ cgit_tag_link(name, title, class, ctx.qry.has_oid ?
+ ctx.qry.oid : ctx.qry.head);
else if (!strcmp(ctx.qry.page, "tree"))
cgit_tree_link(name, title, class, ctx.qry.head,
- ctx.qry.has_sha1 ? ctx.qry.sha1 : NULL,
+ ctx.qry.has_oid ? ctx.qry.oid : NULL,
ctx.qry.path);
else if (!strcmp(ctx.qry.page, "plain"))
cgit_plain_link(name, title, class, ctx.qry.head,
- ctx.qry.has_sha1 ? ctx.qry.sha1 : NULL,
+ ctx.qry.has_oid ? ctx.qry.oid : NULL,
ctx.qry.path);
else if (!strcmp(ctx.qry.page, "blame"))
cgit_blame_link(name, title, class, ctx.qry.head,
- ctx.qry.has_sha1 ? ctx.qry.sha1 : NULL,
+ ctx.qry.has_oid ? ctx.qry.oid : NULL,
ctx.qry.path);
else if (!strcmp(ctx.qry.page, "log"))
cgit_log_link(name, title, class, ctx.qry.head,
- ctx.qry.has_sha1 ? ctx.qry.sha1 : NULL,
+ ctx.qry.has_oid ? ctx.qry.oid : NULL,
ctx.qry.path, ctx.qry.ofs,
ctx.qry.grep, ctx.qry.search,
ctx.qry.showmsg, ctx.qry.follow);
else if (!strcmp(ctx.qry.page, "commit"))
cgit_commit_link(name, title, class, ctx.qry.head,
- ctx.qry.has_sha1 ? ctx.qry.sha1 : NULL,
+ ctx.qry.has_oid ? ctx.qry.oid : NULL,
ctx.qry.path);
else if (!strcmp(ctx.qry.page, "patch"))
cgit_patch_link(name, title, class, ctx.qry.head,
- ctx.qry.has_sha1 ? ctx.qry.sha1 : NULL,
+ ctx.qry.has_oid ? ctx.qry.oid : NULL,
ctx.qry.path);
else if (!strcmp(ctx.qry.page, "refs"))
cgit_refs_link(name, title, class, ctx.qry.head,
- ctx.qry.has_sha1 ? ctx.qry.sha1 : NULL,
+ ctx.qry.has_oid ? ctx.qry.oid : NULL,
ctx.qry.path);
else if (!strcmp(ctx.qry.page, "snapshot"))
cgit_snapshot_link(name, title, class, ctx.qry.head,
- ctx.qry.has_sha1 ? ctx.qry.sha1 : NULL,
+ ctx.qry.has_oid ? ctx.qry.oid : NULL,
ctx.qry.path);
else if (!strcmp(ctx.qry.page, "diff"))
cgit_diff_link(name, title, class, ctx.qry.head,
- ctx.qry.sha1, ctx.qry.sha2,
+ ctx.qry.oid, ctx.qry.oid2,
ctx.qry.path);
else if (!strcmp(ctx.qry.page, "stats"))
cgit_stats_link(name, title, class, ctx.qry.head,
@@ -672,7 +673,7 @@ const struct date_mode *cgit_date_mode(enum date_mode_type type)
static void print_rel_date(time_t t, int tz, double value,
const char *class, const char *suffix)
{
- htmlf("<span class='%s' title='", class);
+ htmlf("<span class='%s' data-ut='%" PRIu64 "' title='", class, (uint64_t)t);
html_attr(show_date(t, tz, cgit_date_mode(DATE_ISO8601)));
htmlf("'>%.0f %s</span>", value, suffix);
}
@@ -767,6 +768,38 @@ static void print_rel_vcs_link(const char *url)
html(" Git repository'/>\n");
}
+static int emit_css_link(struct string_list_item *s, void *arg)
+{
+ /* Do not emit anything if css= is specified. */
+ if (s && *s->string == '\0')
+ return 0;
+
+ html("<link rel='stylesheet' type='text/css' href='");
+ if (s)
+ html_attr(s->string);
+ else
+ html_attr((const char *)arg);
+ html("'/>\n");
+
+ return 0;
+}
+
+static int emit_js_link(struct string_list_item *s, void *arg)
+{
+ /* Do not emit anything if js= is specified. */
+ if (s && *s->string == '\0')
+ return 0;
+
+ html("<script type='text/javascript' src='");
+ if (s)
+ html_attr(s->string);
+ else
+ html_attr((const char *)arg);
+ html("'></script>\n");
+
+ return 0;
+}
+
void cgit_print_docstart(void)
{
char *host = cgit_hosturl();
@@ -786,9 +819,17 @@ void cgit_print_docstart(void)
htmlf("<meta name='generator' content='cgit %s'/>\n", cgit_version);
if (ctx.cfg.robots && *ctx.cfg.robots)
htmlf("<meta name='robots' content='%s'/>\n", ctx.cfg.robots);
- html("<link rel='stylesheet' type='text/css' href='");
- html_attr(ctx.cfg.css);
- html("'/>\n");
+
+ if (ctx.cfg.css.items)
+ for_each_string_list(&ctx.cfg.css, emit_css_link, NULL);
+ else
+ emit_css_link(NULL, "/cgit.css");
+
+ if (ctx.cfg.js.items)
+ for_each_string_list(&ctx.cfg.js, emit_js_link, NULL);
+ else
+ emit_js_link(NULL, "/cgit.js");
+
if (ctx.cfg.favicon) {
html("<link rel='shortcut icon' href='");
html_attr(ctx.cfg.favicon);
@@ -918,10 +959,10 @@ void cgit_add_hidden_formfields(int incl_head, int incl_search,
strcmp(ctx.qry.head, ctx.repo->defbranch))
html_hidden("h", ctx.qry.head);
- if (ctx.qry.sha1)
- html_hidden("id", ctx.qry.sha1);
- if (ctx.qry.sha2)
- html_hidden("id2", ctx.qry.sha2);
+ if (ctx.qry.oid)
+ html_hidden("id", ctx.qry.oid);
+ if (ctx.qry.oid2)
+ html_hidden("id2", ctx.qry.oid2);
if (ctx.qry.showmsg)
html_hidden("showmsg", "1");
@@ -994,7 +1035,7 @@ static void print_header(void)
if (ctx.repo) {
cgit_index_link("index", NULL, NULL, NULL, NULL, 0, 1);
html(" : ");
- cgit_summary_link(ctx.repo->name, ctx.repo->name, NULL, NULL);
+ cgit_summary_link(ctx.repo->name, NULL, NULL, NULL);
if (ctx.env.authenticated) {
html("</td><td class='form'>");
html("<form method='get'>\n");
@@ -1015,7 +1056,13 @@ static void print_header(void)
if (ctx.repo) {
html_txt(ctx.repo->desc);
html("</td><td class='sub right'>");
- html_txt(ctx.repo->owner);
+ if (ctx.repo->owner_filter) {
+ cgit_open_filter(ctx.repo->owner_filter);
+ html_txt(ctx.repo->owner);
+ cgit_close_filter(ctx.repo->owner_filter);
+ } else {
+ html_txt(ctx.repo->owner);
+ }
} else {
if (ctx.cfg.root_desc)
html_txt(ctx.cfg.root_desc);
@@ -1038,20 +1085,20 @@ void cgit_print_pageheader(void)
cgit_summary_link("summary", NULL, hc("summary"),
ctx.qry.head);
cgit_refs_link("refs", NULL, hc("refs"), ctx.qry.head,
- ctx.qry.sha1, NULL);
+ ctx.qry.oid, NULL);
cgit_log_link("log", NULL, hc("log"), ctx.qry.head,
NULL, ctx.qry.vpath, 0, NULL, NULL,
ctx.qry.showmsg, ctx.qry.follow);
if (ctx.qry.page && !strcmp(ctx.qry.page, "blame"))
cgit_blame_link("blame", NULL, hc("blame"), ctx.qry.head,
- ctx.qry.sha1, ctx.qry.vpath);
+ ctx.qry.oid, ctx.qry.vpath);
else
cgit_tree_link("tree", NULL, hc("tree"), ctx.qry.head,
- ctx.qry.sha1, ctx.qry.vpath);
+ ctx.qry.oid, ctx.qry.vpath);
cgit_commit_link("commit", NULL, hc("commit"),
- ctx.qry.head, ctx.qry.sha1, ctx.qry.vpath);
+ ctx.qry.head, ctx.qry.oid, ctx.qry.vpath);
cgit_diff_link("diff", NULL, hc("diff"), ctx.qry.head,
- ctx.qry.sha1, ctx.qry.sha2, ctx.qry.vpath);
+ ctx.qry.oid, ctx.qry.oid2, ctx.qry.vpath);
if (ctx.repo->max_stats)
cgit_stats_link("stats", NULL, hc("stats"),
ctx.qry.head, ctx.qry.vpath);
diff --git a/ui-stats.c b/ui-stats.c
index 7272a61..40ed6c2 100644
--- a/ui-stats.c
+++ b/ui-stats.c
@@ -166,7 +166,7 @@ static void add_commit(struct string_list *authors, struct commit *commit,
struct authorstat *authorstat;
struct string_list *items;
char *tmp;
- struct tm *date;
+ struct tm date;
time_t t;
uintptr_t *counter;
@@ -180,9 +180,9 @@ static void add_commit(struct string_list *authors, struct commit *commit,
authorstat = author->util;
items = &authorstat->list;
t = info->committer_date;
- date = gmtime(&t);
- period->trunc(date);
- tmp = xstrdup(period->pretty(date));
+ gmtime_r(&t, &date);
+ period->trunc(&date);
+ tmp = xstrdup(period->pretty(&date));
item = string_list_insert(items, tmp);
counter = (uintptr_t *)&item->util;
if (*counter)
@@ -215,15 +215,15 @@ static struct string_list collect_stats(const struct cgit_period *period)
int argc = 3;
time_t now;
long i;
- struct tm *tm;
+ struct tm tm;
char tmp[11];
time(&now);
- tm = gmtime(&now);
- period->trunc(tm);
+ gmtime_r(&now, &tm);
+ period->trunc(&tm);
for (i = 1; i < period->count; i++)
- period->dec(tm);
- strftime(tmp, sizeof(tmp), "%Y-%m-%d", tm);
+ period->dec(&tm);
+ strftime(tmp, sizeof(tmp), "%Y-%m-%d", &tm);
argv[2] = xstrdup(fmt("--since=%s", tmp));
if (ctx.qry.path) {
argv[3] = "--";
@@ -241,8 +241,7 @@ static struct string_list collect_stats(const struct cgit_period *period)
memset(&authors, 0, sizeof(authors));
while ((commit = get_revision(&rev)) != NULL) {
add_commit(&authors, commit, period);
- free_commit_buffer(the_repository->parsed_objects, commit);
- free_commit_list(commit->parents);
+ release_commit_memory(the_repository->parsed_objects, commit);
commit->parents = NULL;
}
return authors;
@@ -261,21 +260,21 @@ static void print_combined_authorrow(struct string_list *authors, int from,
struct string_list_item *date;
time_t now;
long i, j, total, subtotal;
- struct tm *tm;
+ struct tm tm;
char *tmp;
time(&now);
- tm = gmtime(&now);
- period->trunc(tm);
+ gmtime_r(&now, &tm);
+ period->trunc(&tm);
for (i = 1; i < period->count; i++)
- period->dec(tm);
+ period->dec(&tm);
total = 0;
htmlf("<tr><td class='%s'>%s</td>", leftclass,
fmt(name, to - from + 1));
for (j = 0; j < period->count; j++) {
- tmp = period->pretty(tm);
- period->inc(tm);
+ tmp = period->pretty(&tm);
+ period->inc(&tm);
subtotal = 0;
for (i = from; i <= to; i++) {
author = &authors->items[i];
@@ -300,20 +299,20 @@ static void print_authors(struct string_list *authors, int top,
struct string_list_item *date;
time_t now;
long i, j, total;
- struct tm *tm;
+ struct tm tm;
char *tmp;
time(&now);
- tm = gmtime(&now);
- period->trunc(tm);
+ gmtime_r(&now, &tm);
+ period->trunc(&tm);
for (i = 1; i < period->count; i++)
- period->dec(tm);
+ period->dec(&tm);
html("<table class='stats'><tr><th>Author</th>");
for (j = 0; j < period->count; j++) {
- tmp = period->pretty(tm);
+ tmp = period->pretty(&tm);
htmlf("<th>%s</th>", tmp);
- period->inc(tm);
+ period->inc(&tm);
}
html("<th>Total</th></tr>\n");
@@ -329,10 +328,10 @@ static void print_authors(struct string_list *authors, int top,
items = &authorstat->list;
total = 0;
for (j = 0; j < period->count; j++)
- period->dec(tm);
+ period->dec(&tm);
for (j = 0; j < period->count; j++) {
- tmp = period->pretty(tm);
- period->inc(tm);
+ tmp = period->pretty(&tm);
+ period->inc(&tm);
date = string_list_lookup(items, tmp);
if (!date)
html("<td>0</td>");
diff --git a/ui-tag.c b/ui-tag.c
index 846d5b1..424bbcc 100644
--- a/ui-tag.c
+++ b/ui-tag.c
@@ -33,7 +33,7 @@ static void print_tag_content(char *buf)
static void print_download_links(char *revname)
{
- html("<tr><th>download</th><td class='sha1'>");
+ html("<tr><th>download</th><td class='oid'>");
cgit_print_snapshot_links(ctx.repo, revname, "<br/>");
html("</td></tr>");
}
@@ -91,7 +91,7 @@ void cgit_print_tag(char *revname)
cgit_close_filter(ctx.repo->email_filter);
html("</td></tr>\n");
}
- html("<tr><td>tagged object</td><td class='sha1'>");
+ html("<tr><td>tagged object</td><td class='oid'>");
cgit_object_link(tag->tagged);
html("</td></tr>\n");
if (ctx.repo->snapshots)
@@ -106,7 +106,7 @@ void cgit_print_tag(char *revname)
html("<tr><td>tag name</td><td>");
html_txt(revname);
html("</td></tr>\n");
- html("<tr><td>tagged object</td><td class='sha1'>");
+ html("<tr><td>tagged object</td><td class='oid'>");
cgit_object_link(obj);
html("</td></tr>\n");
if (ctx.repo->snapshots)
diff --git a/ui-tree.c b/ui-tree.c
index 84eb17d..98ce1ca 100644
--- a/ui-tree.c
+++ b/ui-tree.c
@@ -89,6 +89,7 @@ static void print_object(const struct object_id *oid, const char *path, const ch
enum object_type type;
char *buf;
unsigned long size;
+ bool is_binary;
type = oid_object_info(the_repository, oid, &size);
if (type == OBJ_BAD) {
@@ -103,6 +104,7 @@ static void print_object(const struct object_id *oid, const char *path, const ch
"Error reading object %s", oid_to_hex(oid));
return;
}
+ is_binary = buffer_is_binary(buf, size);
cgit_set_title_from_path(path);
@@ -110,7 +112,7 @@ static void print_object(const struct object_id *oid, const char *path, const ch
htmlf("blob: %s (", oid_to_hex(oid));
cgit_plain_link("plain", NULL, NULL, ctx.qry.head,
rev, path);
- if (ctx.repo->enable_blame) {
+ if (ctx.repo->enable_blame && !is_binary) {
html(") (");
cgit_blame_link("blame", NULL, NULL, ctx.qry.head,
rev, path);
@@ -123,7 +125,7 @@ static void print_object(const struct object_id *oid, const char *path, const ch
return;
}
- if (buffer_is_binary(buf, size))
+ if (is_binary)
print_binary_buffer(buf, size);
else
print_text_buffer(basename, buf, size);
@@ -139,8 +141,7 @@ struct single_tree_ctx {
};
static int single_tree_cb(const struct object_id *oid, struct strbuf *base,
- const char *pathname, unsigned mode, int stage,
- void *cbdata)
+ const char *pathname, unsigned mode, void *cbdata)
{
struct single_tree_ctx *ctx = cbdata;
@@ -185,8 +186,7 @@ static void write_tree_link(const struct object_id *oid, char *name,
tree_ctx.name = NULL;
tree_ctx.count = 0;
- read_tree_recursive(the_repository, tree, "", 0, 1,
- &paths, single_tree_cb, &tree_ctx);
+ read_tree(the_repository, tree, &paths, single_tree_cb, &tree_ctx);
if (tree_ctx.count != 1)
break;
@@ -199,14 +199,16 @@ static void write_tree_link(const struct object_id *oid, char *name,
}
static int ls_item(const struct object_id *oid, struct strbuf *base,
- const char *pathname, unsigned mode, int stage, void *cbdata)
+ const char *pathname, unsigned mode, void *cbdata)
{
struct walk_tree_context *walk_tree_ctx = cbdata;
char *name;
struct strbuf fullpath = STRBUF_INIT;
+ struct strbuf linkpath = STRBUF_INIT;
struct strbuf class = STRBUF_INIT;
enum object_type type;
unsigned long size = 0;
+ char *buf;
name = xstrdup(pathname);
strbuf_addf(&fullpath, "%s%s%s", ctx.qry.path ? ctx.qry.path : "",
@@ -218,8 +220,7 @@ static int ls_item(const struct object_id *oid, struct strbuf *base,
htmlf("<tr><td colspan='3'>Bad object: %s %s</td></tr>",
name,
oid_to_hex(oid));
- free(name);
- return 0;
+ goto cleanup;
}
}
@@ -239,6 +240,21 @@ static int ls_item(const struct object_id *oid, struct strbuf *base,
cgit_tree_link(name, NULL, class.buf, ctx.qry.head,
walk_tree_ctx->curr_rev, fullpath.buf);
}
+ if (S_ISLNK(mode)) {
+ html(" -> ");
+ buf = read_object_file(oid, &type, &size);
+ if (!buf) {
+ htmlf("Error reading object: %s", oid_to_hex(oid));
+ goto cleanup;
+ }
+ strbuf_addbuf(&linkpath, &fullpath);
+ strbuf_addf(&linkpath, "/../%s", buf);
+ strbuf_normalize_path(&linkpath);
+ cgit_tree_link(buf, NULL, class.buf, ctx.qry.head,
+ walk_tree_ctx->curr_rev, linkpath.buf);
+ free(buf);
+ strbuf_release(&linkpath);
+ }
htmlf("</td><td class='ls-size'>%li</td>", size);
html("<td>");
@@ -255,6 +271,8 @@ static int ls_item(const struct object_id *oid, struct strbuf *base,
cgit_blame_link("blame", NULL, "button", ctx.qry.head,
walk_tree_ctx->curr_rev, fullpath.buf);
html("</td></tr>\n");
+
+cleanup:
free(name);
strbuf_release(&fullpath);
strbuf_release(&class);
@@ -294,14 +312,13 @@ static void ls_tree(const struct object_id *oid, const char *path, struct walk_t
}
ls_head();
- read_tree_recursive(the_repository, tree, "", 0, 1,
- &paths, ls_item, walk_tree_ctx);
+ read_tree(the_repository, tree, &paths, ls_item, walk_tree_ctx);
ls_tail();
}
static int walk_tree(const struct object_id *oid, struct strbuf *base,
- const char *pathname, unsigned mode, int stage, void *cbdata)
+ const char *pathname, unsigned mode, void *cbdata)
{
struct walk_tree_context *walk_tree_ctx = cbdata;
@@ -326,7 +343,7 @@ static int walk_tree(const struct object_id *oid, struct strbuf *base,
return 0;
}
}
- ls_item(oid, base, pathname, mode, stage, walk_tree_ctx);
+ ls_item(oid, base, pathname, mode, walk_tree_ctx);
return 0;
}
@@ -370,12 +387,12 @@ void cgit_print_tree(const char *rev, char *path)
walk_tree_ctx.curr_rev = xstrdup(rev);
if (path == NULL) {
- ls_tree(&commit->maybe_tree->object.oid, NULL, &walk_tree_ctx);
+ ls_tree(get_commit_tree_oid(commit), NULL, &walk_tree_ctx);
goto cleanup;
}
- read_tree_recursive(the_repository, commit->maybe_tree, "", 0, 0,
- &paths, walk_tree, &walk_tree_ctx);
+ read_tree(the_repository, repo_get_commit_tree(the_repository, commit),
+ &paths, walk_tree, &walk_tree_ctx);
if (walk_tree_ctx.state == 1)
ls_tail();
else if (walk_tree_ctx.state == 2)
diff --git a/cgit.c b/cgit.c
index c4320f0..57d7097 100644
--- a/cgit.c
+++ b/cgit.c
@@ -142,7 +142,9 @@ static void config_cb(const char *name, const char *value)
else if (!strcmp(name, "root-readme"))
ctx.cfg.root_readme = xstrdup(value);
else if (!strcmp(name, "css"))
- ctx.cfg.css = xstrdup(value);
+ string_list_append(&ctx.cfg.css, xstrdup(value));
+ else if (!strcmp(name, "js"))
+ string_list_append(&ctx.cfg.js, xstrdup(value));
else if (!strcmp(name, "favicon"))
ctx.cfg.favicon = xstrdup(value);
else if (!strcmp(name, "footer"))
@@ -237,9 +239,11 @@ static void config_cb(const char *name, const char *value)
ctx.cfg.max_repodesc_len = atoi(value);
else if (!strcmp(name, "max-blob-size"))
ctx.cfg.max_blob_size = atoi(value);
- else if (!strcmp(name, "max-repo-count"))
+ else if (!strcmp(name, "max-repo-count")) {
ctx.cfg.max_repo_count = atoi(value);
- else if (!strcmp(name, "max-commit-count"))
+ if (ctx.cfg.max_repo_count <= 0)
+ ctx.cfg.max_repo_count = INT_MAX;
+ } else if (!strcmp(name, "max-commit-count"))
ctx.cfg.max_commit_count = atoi(value);
else if (!strcmp(name, "project-list"))
ctx.cfg.project_list = xstrdup(expand_macros(value));
@@ -324,11 +328,11 @@ static void querystring_cb(const char *name, const char *value)
ctx.qry.head = xstrdup(value);
ctx.qry.has_symref = 1;
} else if (!strcmp(name, "id")) {
- ctx.qry.sha1 = xstrdup(value);
- ctx.qry.has_sha1 = 1;
+ ctx.qry.oid = xstrdup(value);
+ ctx.qry.has_oid = 1;
} else if (!strcmp(name, "id2")) {
- ctx.qry.sha2 = xstrdup(value);
- ctx.qry.has_sha1 = 1;
+ ctx.qry.oid2 = xstrdup(value);
+ ctx.qry.has_oid = 1;
} else if (!strcmp(name, "ofs")) {
ctx.qry.ofs = atoi(value);
} else if (!strcmp(name, "path")) {
@@ -376,7 +380,6 @@ static void prepare_context(void)
ctx.cfg.case_sensitive_sort = 1;
ctx.cfg.branch_sort = 0;
ctx.cfg.commit_sort = 0;
- ctx.cfg.css = "/cgit.css";
ctx.cfg.logo = "/cgit.png";
ctx.cfg.favicon = "/favicon.ico";
ctx.cfg.local_time = 0;
@@ -428,7 +431,7 @@ static void prepare_context(void)
ctx.page.modified = time(NULL);
ctx.page.expires = ctx.page.modified;
ctx.page.etag = NULL;
- string_list_init(&ctx.cfg.mimetypes, 1);
+ string_list_init_dup(&ctx.cfg.mimetypes);
if (ctx.env.script_name)
ctx.cfg.script_name = xstrdup(ctx.env.script_name);
if (ctx.env.query_string)
@@ -507,9 +510,11 @@ static inline void parse_readme(const char *readme, char **filename, char **ref,
/* Check if the readme is tracked in the git repo. */
colon = strchr(readme, ':');
if (colon && strlen(colon) > 1) {
- /* If it starts with a colon, we want to use
- * the default branch */
- if (colon == readme && repo->defbranch)
+ /* If it starts with a colon, we want to use head given
+ * from query or the default branch */
+ if (colon == readme && ctx.qry.head)
+ *ref = xstrdup(ctx.qry.head);
+ else if (colon == readme && repo->defbranch)
*ref = xstrdup(repo->defbranch);
else
*ref = xstrndup(readme, colon - readme);
@@ -992,9 +997,9 @@ static void cgit_parse_args(int argc, const char **argv)
} else if (skip_prefix(argv[i], "--head=", &arg)) {
ctx.qry.head = xstrdup(arg);
ctx.qry.has_symref = 1;
- } else if (skip_prefix(argv[i], "--sha1=", &arg)) {
- ctx.qry.sha1 = xstrdup(arg);
- ctx.qry.has_sha1 = 1;
+ } else if (skip_prefix(argv[i], "--oid=", &arg)) {
+ ctx.qry.oid = xstrdup(arg);
+ ctx.qry.has_oid = 1;
} else if (skip_prefix(argv[i], "--ofs=", &arg)) {
ctx.qry.ofs = atoi(arg);
} else if (skip_prefix(argv[i], "--scan-tree=", &arg) ||
@@ -1037,7 +1042,7 @@ static int calc_ttl(void)
if (!strcmp(ctx.qry.page, "snapshot"))
return ctx.cfg.cache_snapshot_ttl;
- if (ctx.qry.has_sha1)
+ if (ctx.qry.has_oid)
return ctx.cfg.cache_static_ttl;
if (ctx.qry.has_symref)
diff --git a/ui-snapshot.c b/ui-snapshot.c
index 9461d51..18361a6 100644
--- a/ui-snapshot.c
+++ b/ui-snapshot.c
@@ -13,32 +13,32 @@
static int write_archive_type(const char *format, const char *hex, const char *prefix)
{
- struct argv_array argv = ARGV_ARRAY_INIT;
+ struct strvec argv = STRVEC_INIT;
const char **nargv;
int result;
- argv_array_push(&argv, "snapshot");
- argv_array_push(&argv, format);
+ strvec_push(&argv, "snapshot");
+ strvec_push(&argv, format);
if (prefix) {
struct strbuf buf = STRBUF_INIT;
strbuf_addstr(&buf, prefix);
strbuf_addch(&buf, '/');
- argv_array_push(&argv, "--prefix");
- argv_array_push(&argv, buf.buf);
+ strvec_push(&argv, "--prefix");
+ strvec_push(&argv, buf.buf);
strbuf_release(&buf);
}
- argv_array_push(&argv, hex);
+ strvec_push(&argv, hex);
/*
* Now we need to copy the pointers to arguments into a new
* structure because write_archive will rearrange its arguments
* which may result in duplicated/missing entries causing leaks
- * or double-frees in argv_array_clear.
+ * or double-frees in strvec_clear.
*/
- nargv = xmalloc(sizeof(char *) * (argv.argc + 1));
- /* argv_array guarantees a trailing NULL entry. */
- memcpy(nargv, argv.argv, sizeof(char *) * (argv.argc + 1));
+ nargv = xmalloc(sizeof(char *) * (argv.nr + 1));
+ /* strvec guarantees a trailing NULL entry. */
+ memcpy(nargv, argv.v, sizeof(char *) * (argv.nr + 1));
- result = write_archive(argv.argc, nargv, NULL, the_repository, NULL, 0);
- argv_array_clear(&argv);
+ result = write_archive(argv.nr, nargv, NULL, the_repository, NULL, 0);
+ strvec_clear(&argv);
free(nargv);
return result;
}
@@ -79,18 +79,32 @@ static int write_tar_bzip2_archive(const char *hex, const char *prefix)
return write_compressed_tar_archive(hex, prefix, argv);
}
+static int write_tar_lzip_archive(const char *hex, const char *prefix)
+{
+ char *argv[] = { "lzip", NULL };
+ return write_compressed_tar_archive(hex, prefix, argv);
+}
+
static int write_tar_xz_archive(const char *hex, const char *prefix)
{
char *argv[] = { "xz", NULL };
return write_compressed_tar_archive(hex, prefix, argv);
}
+static int write_tar_zstd_archive(const char *hex, const char *prefix)
+{
+ char *argv[] = { "zstd", "-T0", NULL };
+ return write_compressed_tar_archive(hex, prefix, argv);
+}
+
const struct cgit_snapshot_format cgit_snapshot_formats[] = {
/* .tar must remain the 0 index */
{ ".tar", "application/x-tar", write_tar_archive },
{ ".tar.gz", "application/x-gzip", write_tar_gzip_archive },
{ ".tar.bz2", "application/x-bzip2", write_tar_bzip2_archive },
+ { ".tar.lz", "application/x-lzip", write_tar_lzip_archive },
{ ".tar.xz", "application/x-xz", write_tar_xz_archive },
+ { ".tar.zst", "application/x-zstd", write_tar_zstd_archive },
{ ".zip", "application/x-zip", write_zip_archive },
{ NULL }
};